/** @internal */
// baseKey includes "." at the end so that we don't have to check if it's the top level or not
const getFields = ({ fields, expand }, baseKey = '') => {
    const fieldsAtThisLevel = fields
        ? fields.map((field) => `${baseKey}${field}`).join(',')
        : expand
            ? `${baseKey}*`
            : baseKey.slice(0, -1); // if it doesn't expand any further, there's no need to add ".*"
    if (expand) {
        // check if any of the expand has fields specified
        if (JSON.stringify(expand).includes('"fields"')) {
            const expandFields = expand.map((exp) => {
                return getFields(exp, `${baseKey}expand.${exp.key}.`);
            });
            return `${fieldsAtThisLevel},${expandFields.join(',')}`;
        }
        // if not, add ".*" to include all fields at this level and below
        return `${fieldsAtThisLevel},${baseKey}expand.*`;
    }
    return `${fieldsAtThisLevel}`;
};
/** @internal */
const getExpand = (option, baseKey = '') => {
    return option
        .map(({ key, expand }) => {
        return expand ? `${getExpand(expand, `${baseKey}${key}.`)}` : `${baseKey}${key}`;
    })
        .join(',');
};
/** @internal */
export const processFilterAndSort = (filter) => {
    return typeof filter === 'function'
        ? filter({
            $: (str, ...values) => {
                return values.reduce((acc, val, i) => acc + val + str[i + 1], str[0]);
            },
        })
        : filter;
};
/** @internal */
export const processOptions = (option) => {
    if (!option ||
        // if these are already of type `string`, it's already been processed and the method was called internally
        typeof option.fields === 'string' ||
        typeof option.expand === 'string') {
        return option;
    }
    const expand = option.expand ? getExpand(option.expand) : undefined;
    const fields = JSON.stringify(option).includes('"fields"')
        ? getFields(option)
        : undefined;
    const filter = processFilterAndSort(option.filter);
    const sort = processFilterAndSort(option.sort);
    const res = { ...option, fields, expand, filter, sort };
    // remove undefined values. the sdk will throw error if left unhandled
    for (const [key, value] of Object.entries(res)) {
        // `null` is a valid value for `requestKey` and should not be removed
        if (value === undefined) {
            delete res[key];
        }
    }
    return res;
};
