import { RecordService } from 'pocketbase';
import type { ListResult, RecordSubscription, UnsubscribeFunc } from 'pocketbase';
import type { Options } from '../helpers/options.js';
import type { BodyParams } from '../helpers/body-params.js';
import type { FilterHelper } from '../helpers/filter.js';
import type { PocketBaseTS } from '../Client.js';
import type { PBResponseType } from '../helpers/response.js';
import type { SchemaDeclaration } from '../schema.js';
export declare class RecordServiceTS<TSchema extends SchemaDeclaration, TKey extends keyof TSchema, TMaxDepth extends number, _Type = TSchema[TKey]['type'], _ListOptions extends Options<TSchema, TKey, TMaxDepth, true> = Options<TSchema, TKey, TMaxDepth, true>, _ViewOptions extends Options<TSchema, TKey, TMaxDepth, false> = Options<TSchema, TKey, TMaxDepth, false>> extends RecordService {
    constructor(client: PocketBaseTS<any>, idOrName: TKey & string);
    subscribe<const TOption extends _ViewOptions>(topic: string, callback: (data: RecordSubscription<PBResponseType<TSchema, TKey, TOption, TMaxDepth>>) => void): Promise<UnsubscribeFunc>;
    subscribe<const TOption extends _ViewOptions>(topic: string, callback: (data: RecordSubscription<PBResponseType<TSchema, TKey, TOption, TMaxDepth>>) => void, options: TOption): Promise<UnsubscribeFunc>;
    getFullList<const TOptions extends _ListOptions>(options?: TOptions): Promise<Array<PBResponseType<TSchema, TKey, TOptions, TMaxDepth>>>;
    getList(page: number, perPage: number): Promise<ListResult<_Type>>;
    getList<const TOptions extends _ListOptions>(page: number, perPage: number, options: TOptions): Promise<ListResult<PBResponseType<TSchema, TKey, TOptions, TMaxDepth>>>;
    getFirstListItem(filter: string | FilterHelper<TSchema, TKey, TMaxDepth, true>): Promise<_Type>;
    getFirstListItem<const TOptions extends _ListOptions>(filter: string | FilterHelper<TSchema, TKey, TMaxDepth, true>, options: TOptions): Promise<PBResponseType<TSchema, TKey, TOptions, TMaxDepth>>;
    getOne(id: string): Promise<_Type>;
    getOne<const TOptions extends _ViewOptions>(id: string, options: TOptions): Promise<PBResponseType<TSchema, TKey, TOptions, TMaxDepth>>;
    create(bodyParams?: Partial<Omit<_Type, 'created' | 'updated'>> | {
        [key: string]: any;
    } | FormData): Promise<_Type>;
    create<const TOptions extends _ViewOptions>(bodyParams: Partial<Omit<_Type, 'created' | 'updated'>> | {
        [key: string]: any;
    } | FormData, options: TOptions): Promise<PBResponseType<TSchema, TKey, TOptions, TMaxDepth>>;
    update(id: string, bodyParams?: BodyParams<_Type> | {
        [key: string]: any;
    } | FormData): Promise<_Type>;
    update<const TOptions extends _ViewOptions>(id: string, bodyParams: BodyParams<_Type> | {
        [key: string]: any;
    } | FormData, options: TOptions): Promise<PBResponseType<TSchema, TKey, TOptions, TMaxDepth>>;
}
